#include <signal.h>
#include <time.h>
#include <stdio.h>
#include <sched.h>

#define PERIOD_SIG      SIGRTMIN
#define PERIOD_SEC      0
#define PERIOD_NSEC     1000000 // 1ms


int main(int argc, char *argv[])
{
	sigset_t mask;
	timer_t timerid;
	struct itimerspec its;
	struct sigevent evp = {
		.sigev_notify = SIGEV_SIGNAL,
		.sigev_signo = PERIOD_SIG,
	};
	struct sched_param sched = {
		.sched_priority = sched_get_priority_max(SCHED_FIFO),
	};
	siginfo_t si;
	int sig;

	if (sched_setscheduler(0, SCHED_FIFO, &sched))
	{
		perror("sched_setscheduler()");
		return 1;
	}

	/* Bloque le signal PERIOD_SIG (la réception de PERIOD_SIG n'entraîne plus
	   l'appel d'un handler déclaré avec sigaction) */
	if (sigemptyset(&mask) ||
	    sigaddset(&mask, PERIOD_SIG) ||
	    sigprocmask(SIG_BLOCK, &mask, NULL))
		return 2;

	if (timer_create(CLOCK_REALTIME, &evp, &timerid))
		return 3;

	its.it_value.tv_sec = PERIOD_SEC;
	its.it_value.tv_nsec = PERIOD_NSEC;
	its.it_interval.tv_sec = PERIOD_SEC;
	its.it_interval.tv_nsec = PERIOD_NSEC;
	if (timer_settime(timerid, 0, &its, NULL))
		return 4;

	while (1)
	{
		if ((sig = sigwaitinfo(&mask, &si)) < 0)
			return 5;
		if (sig != PERIOD_SIG)
			continue;
		/* Periodic task... */
	}

	return 0;
}
